import { QuintError } from './quintError';
/**
 * An error message whose locations have been resolved.
 */
export interface ErrorMessage {
    explanation: string;
    locs: Loc[];
}
export interface Loc {
    source: string;
    start: {
        line: number;
        col: number;
        index: number;
    };
    end?: {
        line: number;
        col: number;
        index: number;
    };
}
/**
 * Map an identifier to the corresponding location in the source map, if possible.
 * @param sourceMap the source map
 * @param id the identifier to map
 * @returns the location, if found in the map, or the unknown location
 */
export declare function sourceIdToLoc(sourceMap: Map<bigint, Loc>, id: bigint): Loc;
export declare function fromQuintError(sourceMap: Map<bigint, Loc>): (_: QuintError) => ErrorMessage;
