import { QuintModule } from './ir/quintIr';
/**
 * A documentation entry for a definition, compatible with LSP responses for signature help
 */
export interface DocumentationEntry {
    name: string;
    signature: string;
    documentation?: string;
}
/**
 * Produces a documentation map for the modules' definitions
 *
 * @param quintModule the module for which documentation should be produced
 * @returns a map of definition names to their documentation
 */
export declare function produceDocs(quintModule: QuintModule): Map<string, DocumentationEntry>;
/**
 * Produces a documentation map for the modules' definitions
 *
 * @param quintModule the module for which documentation should be produced
 * @returns a map of definition ids to their documentation
 */
export declare function produceDocsById(quintModule: QuintModule): Map<bigint, DocumentationEntry>;
/**
 * Formats a documentation entry to Markdown
 *
 * @param entry the documentation entry to be formatted
 * @returns a string with the entry's label as header and documentation as body
 */
export declare function toMarkdown(entry: DocumentationEntry): string;
