/**
 * A structure to track errors across nested function calls in a tree, along
 * with helpful build functions
 *
 * @author Gabriela Moreira
 *
 * @module
 */
export interface ErrorTree {
    message?: string;
    location: string;
    children: ErrorTree[];
}
export type Error = ErrorTree | ErrorTree[] | string;
export declare function buildErrorTree(location: string, errors: Error): ErrorTree;
export declare function buildErrorLeaf(location: string, message: string): ErrorTree;
/**
 * Formats the string representation of an error tree
 *
 * @param e the ErrorTree to be formatted
 *
 * @returns a multiline string with the pretty printed error tree
 */
export declare function errorTreeToString(e: ErrorTree): string;
