import { QuintDef, QuintEx, QuintLambdaParameter, QuintModule } from './quintIr';
import { QuintType } from './quintTypes';
/**
 * Finds a quint expression with a given id in a list of modules
 *
 * @param modules the modules in which to search for the expression
 * @param id the id to be searched for
 *
 * @returns a quint expression with the given id, or undefined if no expression is found
 */
export declare function findExpressionWithId(modules: QuintModule[], id: bigint): QuintEx | undefined;
/**
 * Finds a quint type with a given id in a list of modules
 *
 * @param modules the modules in which to search for the type
 * @param id the id to be searched for
 *
 * @returns a quint type with the given id, or undefined if no type is found
 */
export declare function findTypeWithId(modules: QuintModule[], id: bigint): QuintType | undefined;
/**
 * Finds a quint definition with a given id in a list of modules
 *
 * @param modules the modules in which to search for the definition
 * @param id the id to be searched for
 *
 * @returns a quint definition with the given id, or undefined if no expression is found
 */
export declare function findDefinitionWithId(modules: QuintModule[], id: bigint): QuintDef | undefined;
/**
 * Find a quint parameter with a given id in a list of modules
 *
 * @param modules the modules in which to search for the parameter
 * @param id the id to be searched for
 *
 * @returns a quint lambda parameter with the given id, or undefined if no parameter is found
 */
export declare function findParameterWithId(modules: QuintModule[], id: bigint): QuintLambdaParameter | undefined;
