/**
 * Representation for quint errors
 *
 * @author Gabriela Moreira
 *
 * @module
 */
import { ErrorTree } from './errorTree';
export interface QuintError {
    code: ErrorCode;
    message: string;
    reference?: bigint;
    data?: QuintErrorData;
}
export type ErrorCode = 'QNT000' | 'QNT001' | 'QNT002' | 'QNT003' | 'QNT004' | 'QNT005' | 'QNT006' | 'QNT007' | 'QNT008' | 'QNT009' | 'QNT012' | 'QNT013' | 'QNT014' | 'QNT098' | 'QNT099' | 'QNT101' | 'QNT102' | 'QNT200' | 'QNT201' | 'QNT202' | 'QNT203' | 'QNT204' | 'QNT205' | 'QNT206' | 'QNT404' | 'QNT405' | 'QNT406' | 'QNT407' | 'QNT408' | 'QNT500' | 'QNT501' | 'QNT502' | 'QNT503' | 'QNT504' | 'QNT505' | 'QNT506' | 'QNT507' | 'QNT508' | 'QNT509' | 'QNT510' | 'QNT511' | 'QNT512' | 'QNT513' | 'QNT514';
export interface QuintErrorData {
    fix?: QuintErrorFix;
    trace?: ErrorTree;
}
export type QuintErrorFix = {
    kind: 'replace';
    original: string;
    replacement: string;
};
/** Formats a Quint error as a string
 *
 * @param err the error to be formatted
 *
 * @returns a string representation of the error
 */
export declare function quintErrorToString(err: QuintError): string;
