/**
 * Core logic of verification
 *
 * @author Shon Feder, Informal Systems, 2024
 * @author Igor Konnov, konnov.phd, 2024
 *
 * @module
 */
import { ApalacheResult, ServerEndpoint } from './apalache';
/**
 * Verifies the configuration `config` by model checking it with the Apalache server
 *
 * @param serverEndpoint
 *   a server endpoint
 *
 * @param config
 *   an apalache configuration. See https://github.com/apalache-mc/apalache/blob/main/mod-infra/src/main/scala/at/forsyte/apalache/infra/passes/options.scala#L255
 *
 * @returns right(void) if verification succeeds, or left(err) explaining the failure
 */
export declare function verify(serverEndpoint: ServerEndpoint, apalacheVersion: string, config: any, verbosityLevel: number): Promise<ApalacheResult<void>>;
