"use strict";
/* ----------------------------------------------------------------------------------
 * Copyright 2024 Informal Systems
 * Licensed under the Apache License, Version 2.0.
 * See LICENSE in the project root for license information.
 * --------------------------------------------------------------------------------- */
Object.defineProperty(exports, "__esModule", { value: true });
exports.verify = void 0;
/**
 * Core logic of verification
 *
 * @author Shon Feder, Informal Systems, 2024
 * @author Igor Konnov, konnov.phd, 2024
 *
 * @module
 */
// TODO This module should subsume the pure logic from the verify commaind in cliCommand
const apalache_1 = require("./apalache");
/**
 * Verifies the configuration `config` by model checking it with the Apalache server
 *
 * @param serverEndpoint
 *   a server endpoint
 *
 * @param config
 *   an apalache configuration. See https://github.com/apalache-mc/apalache/blob/main/mod-infra/src/main/scala/at/forsyte/apalache/infra/passes/options.scala#L255
 *
 * @returns right(void) if verification succeeds, or left(err) explaining the failure
 */
async function verify(serverEndpoint, apalacheVersion, config, verbosityLevel) {
    const connectionResult = await (0, apalache_1.connect)(serverEndpoint, apalacheVersion, verbosityLevel);
    return connectionResult.asyncChain(conn => conn.check(config));
}
exports.verify = verify;
//# sourceMappingURL=quintVerifier.js.map