/**
 * Constraint generation for Quint's type system
 *
 * @author Gabriela Moreira
 *
 * @module
 */
import { IRVisitor } from '../ir/IRVisitor';
import { QuintApp, QuintAssume, QuintBool, QuintConst, QuintDeclaration, QuintEx, QuintInstance, QuintInt, QuintLambda, QuintLet, QuintName, QuintOpDef, QuintStr, QuintVar } from '../ir/quintIr';
import { Either } from '@sweet-monads/either';
import { ErrorTree } from '../errorTree';
import { Constraint, QuantifiedVariables, TypeScheme } from './base';
import { Substitutions } from './substitutions';
import { LookupTable } from '../names/base';
import { FreshVarGenerator } from '../FreshVarGenerator';
export type SolvingFunctionType = (_table: LookupTable, _constraint: Constraint) => Either<Map<bigint, ErrorTree>, Substitutions>;
export declare class ConstraintGeneratorVisitor implements IRVisitor {
    constructor(solvingFunction: SolvingFunctionType, table: LookupTable, types?: Map<bigint, TypeScheme>);
    protected types: Map<bigint, TypeScheme>;
    protected errors: Map<bigint, ErrorTree>;
    protected freshVarGenerator: FreshVarGenerator;
    protected table: LookupTable;
    private constraints;
    private solvingFunction;
    private builtinSignatures;
    protected tvs: Map<bigint, QuantifiedVariables>;
    protected typesInScope: Map<bigint, TypeScheme>;
    private location;
    getResult(): [Map<bigint, ErrorTree>, Map<bigint, TypeScheme>];
    enterExpr(e: QuintEx): void;
    exitVar(e: QuintVar): void;
    exitConst(e: QuintConst): void;
    exitInstance(def: QuintInstance): void;
    exitName(e: QuintName): void;
    exitLiteral(e: QuintBool | QuintInt | QuintStr): void;
    exitApp(e: QuintApp): void;
    enterLambda(expr: QuintLambda): void;
    exitLambda(e: QuintLambda): void;
    exitLet(e: QuintLet): void;
    exitDecl(_def: QuintDeclaration): void;
    enterOpDef(def: QuintOpDef): void;
    exitOpDef(def: QuintOpDef): void;
    enterAssume(e: QuintAssume): void;
    exitAssume(e: QuintAssume): void;
    private addToResults;
    private fetchResult;
    private solveConstraints;
    private typeForName;
    private newInstance;
    private freeNamesInScope;
}
